/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 */
package com.jcg;

import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 *
 * @author omozegieaziegbe
 */
public class SshMysqlConnection {

    private final static Logger LOGGER = Logger.getLogger(SshMysqlConnection.class.getName());
    
    private static final String SSH_HOST = "ssh.example.com";
    private static final int SSH_PORT = 22;
    private static final String SSH_USER = "ssh_user";
    private static final String SSH_PRIVATE_KEY = "/path/to/private/key"; // Or use a password

    private static final String REMOTE_MYSQL_HOST = "127.0.0.1";
    private static final int REMOTE_MYSQL_PORT = 3306;
    private static final String LOCAL_PORT = "3307"; // Local forwarding port

    private static final String MYSQL_USER = "db_user";
    private static final String MYSQL_PASSWORD = "db_password";
    private static final String DATABASE_NAME = "testdb";

    private Session session;

    public void connectAndQueryDatabase() {
        try {
            // Establish the SSH Tunnel
            JSch jsch = new JSch();
            jsch.addIdentity(SSH_PRIVATE_KEY);

            session = jsch.getSession(SSH_USER, SSH_HOST, SSH_PORT);
            session.setConfig("StrictHostKeyChecking", "no");
            session.connect();

            System.out.println("SSH connection established.");
            
            // Set up port forwarding
            int assignedPort = session.setPortForwardingL(Integer.parseInt(LOCAL_PORT), REMOTE_MYSQL_HOST, REMOTE_MYSQL_PORT);
            System.out.println("Port forwarding established: localhost:" + assignedPort + " -> " + REMOTE_MYSQL_HOST + ":" + REMOTE_MYSQL_PORT);

            // Connect to MySQL database
            String jdbcUrl = "jdbc:mysql://localhost:" + assignedPort + "/" + DATABASE_NAME;
            try (Connection connection = DriverManager.getConnection(jdbcUrl, MYSQL_USER, MYSQL_PASSWORD)) {
                System.out.println("Connected to the MySQL database.");
                queryDatabase(connection);
            }

        } catch (JSchException | SQLException e) {
            LOGGER.log(Level.SEVERE, e.getMessage()); 
        } finally {
            if (session != null && session.isConnected()) {
                session.disconnect();
                System.out.println("SSH connection closed.");
            }
        }
    }

    private void queryDatabase(Connection connection) {
        String query = "SELECT * FROM users";
        try (PreparedStatement statement = connection.prepareStatement(query); ResultSet resultSet = statement.executeQuery()) {

            while (resultSet.next()) {
                System.out.println("User ID: " + resultSet.getInt("id"));
                System.out.println("Username: " + resultSet.getString("username"));
            }

        } catch (Exception e) {
           LOGGER.log(Level.SEVERE, e.getMessage()); 
        }
    }

    public static void main(String[] args) {
        SshMysqlConnection app = new SshMysqlConnection();
        app.connectAndQueryDatabase();
    }
}
